package sun.nio.ch;

import java.lang.reflect.Field;
import java.io.IOException;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Constructor;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.reflect.AccessibleObject;

class Reflect
{
    private Reflect() {
    }
    
    private static void setAccessible(final AccessibleObject accessibleObject) {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                accessibleObject.setAccessible(true);
                return null;
            }
        });
    }
    
    static Constructor<?> lookupConstructor(String className, Class<?>[] parameterTypes) {
        try {
            Constructor<Object> declaredConstructor = Class.forName(className).getDeclaredConstructor(parameterTypes);
            setAccessible(declaredConstructor);
            return declaredConstructor;
        }
        catch (ClassNotFoundException | NoSuchMethodException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static Object invoke(Constructor<?> constructor, Object[] initargs) {
        try {
            return constructor.newInstance(initargs);
        }
        catch (InstantiationException | IllegalAccessException | InvocationTargetException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static Method lookupMethod(String className, String name, Class... parameterTypes) {
        try {
            Method declaredMethod = Class.forName(className).getDeclaredMethod(name, parameterTypes);
            setAccessible(declaredMethod);
            return declaredMethod;
        }
        catch (ClassNotFoundException | NoSuchMethodException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static Object invoke(Method method, Object obj, Object[] args) {
        try {
            return method.invoke(obj, args);
        }
        catch (IllegalAccessException | InvocationTargetException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static Object invokeIO(Method method, Object obj, Object[] args) throws IOException {
        try {
            return method.invoke(obj, args);
        }
        catch (IllegalAccessException ex) {
            throw new Reflect.ReflectionError(ex);
        }
        catch (InvocationTargetException ex2) {
            if (IOException.class.isInstance(ex2.getCause())) {
                throw (IOException)ex2.getCause();
            }
            throw new Reflect.ReflectionError(ex2);
        }
    }
    
    static Field lookupField(String className, String name) {
        try {
            Field declaredField = Class.forName(className).getDeclaredField(name);
            setAccessible(declaredField);
            return declaredField;
        }
        catch (ClassNotFoundException | NoSuchFieldException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static Object get(Object obj, Field field) {
        try {
            return field.get(obj);
        }
        catch (IllegalAccessException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static Object get(Field field) {
        return get(null, field);
    }
    
    static void set(Object obj, Field field, Object value) {
        try {
            field.set(obj, value);
        }
        catch (IllegalAccessException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static void setInt(Object obj, Field field, int i) {
        try {
            field.setInt(obj, i);
        }
        catch (IllegalAccessException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    static void setBoolean(Object obj, Field field, boolean z) {
        try {
            field.setBoolean(obj, z);
        }
        catch (IllegalAccessException ex) {
            throw new Reflect.ReflectionError(ex);
        }
    }
    
    private static class ReflectionError extends Error
    {
        private static final long serialVersionUID = -8659519328078164097L;
        
        ReflectionError(Throwable cause) {
            super(cause);
        }
    }
}
